!==============================================================================================================================================================
!  计算自由电子的超流权重
!  H(k) = t(cos kx + cos ky)  正方晶格最近邻
!==============================================================================================================================================================
module code_param
    implicit none
    integer, parameter :: dp = kind(1.0)
    real(dp),parameter::pi = acos(-1.0_dp)
    complex(dp),parameter::im = (0.,1.)                 !   Imagine unit  
    integer hn,hnn,numk_bz,kn,numk_FS,Un
    real(dp) kbt,delta_E,delta_k,engcut,sigma,Umax     
    real(dp) t1,mu  ! 哈密顿量参数
    parameter(t1 = 1.0,mu = 1.0)
    parameter(hn = 2,hnn = hn/2,kn = 2e2 ,Un = 100,Umax = 1.0,kbt = 1e-5,delta_E = 1e-5,engcut = 100,sigma = 1e-5,delta_k = 1.0/kn)  ! hn: 哈密顿量维度
    real(dp),allocatable::BZklist(:,:)
end module code_param
!==============================================================================================================================================================
program main
    use code_param
    use mpi
    implicit none
    integer numcore,indcore,ierr,nki,nkf
    integer i0,i1,i2
    real(dp) kx,ky,U0_mpi(Un),U0_list(Un),U0
    complex(dp) da_mpi(Un),db_mpi(Un),da_list(Un),db_list(Un)
    !-----------------------------------------------------------------------------------------------------------------
    !#######################################         并行计算设置      #######################################
    call MPI_INIT(ierr)     ! 初始化进程
    call MPI_COMM_RANK(MPI_COMM_WORLD, indcore, ierr) ! 得到本进程在通信空间中的rank值,即在组中的逻辑编号(该indcore为0到numcore-1间的整数,相当于进程的ID。)
    call MPI_COMM_SIZE(MPI_COMM_WORLD, numcore, ierr) !获得进程数量,用numcoer保存
    ! 并行循环分拆
    nki = floor(indcore * (1.0 * Un)/numcore) + 1
    nkf = floor((indcore + 1) * (1.0 * Un)/numcore)
    !------------------------------------------------------------------------------------------------------------------
    ! 预设布里渊区撒点
    call squareBZ()  
    !------------------------------------------------------------------------------------------------------------------
    if(indcore.eq.0)then
        call Fermi_surface()
    end if
    !------------------------------------------------------------------------------------------------------------------
    ! 测试超流权重随相互作用U的变化，同时在每个U下面要自洽超导序参量
    do i0 = nki,nkf
        U0 = Umax/Un * (i0 - 1)   ! 改变相互作用强度
        U0_mpi(i0) = U0
        call gap_equation(U0,da_mpi(i0)) !  自洽序参量
    end do

    call MPI_Barrier(MPI_COMM_WORLD,ierr)   ! 等所有核心都计算完成r)
    call MPI_Reduce(U0_mpi, U0_list, Un, MPI_REAL, MPI_SUM, 0, MPI_COMM_WORLD,ierr)
    call MPI_Reduce(da_mpi, da_list, Un, MPI_COMPLEX, MPI_SUM, 0, MPI_COMM_WORLD,ierr)

    if(indcore.eq.0)then
        ! 数据读写

        open(32,file = "delta-normal.dat")  ! 序参量
        do i0 = 1,Un
            write(32,"(9F15.8)")U0_list(i0),real(da_list(i0)),aimag(da_list(i0))
        enddo
        close(32)
    endif
    call MPI_Finalize(ierr)
    stop
end program main

!==============================================================================================================================================================
subroutine gap_equation(U0,delta_A)
    !  自洽序参量,然后通过参数返回
    use code_param
    implicit none
    complex(dp) delta_A,delta_B
    complex(dp) Ham(hn,hn),matvec(hn,hn),new_deltaA,old_deltaA,new_deltaB,old_deltaB,mat_temp(hn,hn)
    real(dp) matval(hn),kx,ky,diff_delta,diff_A,diff_B,ones_mat(hn,hn),U0   ! diff_delta : 控制序参量自洽精度
    integer ik0,ie0,ref,matdim
    parameter(diff_delta = 1e-5,matdim = hn)
    real(dp),external::fermi
    ! 初猜化学势
    old_deltaA = 0.1
    diff_A = diff_delta + 1.0  ! 使循环能进入
    ! 自洽循环
    do while (diff_A > diff_delta)
        ! open(25,file = "self_loop.dat",access = 'append')
        ! 下面要重新自洽序参量
        new_deltaA = 0.0
        do ik0 = 1,size(BZklist,2)   ! 遍历布里渊区点
            kx = BZklist(1,ik0)
            ky = BZklist(2,ik0)
            call matset_BdG_SC(kx,ky,Ham,old_deltaA)
            call diagonalize_Hermitian_matrix(matdim,Ham,matvec,matval)

            ! Ref: Superconductivity in geometrically and topologically nontrivial lattice models
            ! 该公式需要 U > 0
            ones_mat = 0.0
            do ie0 = 1,hn
                ones_mat(ie0,ie0) = fermi(matval(ie0))
            end do
            mat_temp = matmul(matmul(matvec,ones_mat),transpose(conjg(matvec)))
            new_deltaA = new_deltaA + mat_temp(1,2)

        end do
        ! 自洽得到新的序参(归一化处理)
        new_deltaA = -U0 * new_deltaA * delta_k**2    ! 吸引相互作用才能自洽出超导
        ! 给定差异来停止自洽过程
        diff_A = abs(new_deltaA - old_deltaA)
        ! 重新赋值继续自洽
        old_deltaA = new_deltaA
    end do
    !  返回自洽收敛后的序参量
    delta_A = new_deltaA
    return
end subroutine
!==============================================================================================================================================================
subroutine gap_equation_new(U0,delta_A)
    !  自洽序参量,然后通过参数返回
    use code_param
    implicit none
    complex(dp) delta_A,delta_B
    complex(dp) Ham(hn,hn),matvec(hn,hn),new_deltaA,old_deltaA,new_deltaB,old_deltaB,mat_temp(hn,hn)
    real(dp) matval(hn),kx,ky,diff_delta,diff_A,diff_B,ones_mat(hn,hn),U0   ! diff_delta : 控制序参量自洽精度
    integer ik0,ie0,ref,matdim
    parameter(diff_delta = 1e-5,matdim = hn)
    real(dp),external::fermi
    ! 初猜化学势
    old_deltaA = 0.1
    diff_A = diff_delta + 1.0  ! 使循环能进入
    ! 自洽循环
    do while (diff_A > diff_delta)
        ! open(25,file = "self_loop.dat",access = 'append')
        ! 下面要重新自洽序参量
        new_deltaA = 0.0
        do ik0 = 1,size(BZklist,2)   ! 遍历布里渊区点
            kx = BZklist(1,ik0)
            ky = BZklist(2,ik0)
            call matset_BdG_SC(kx,ky,Ham,old_deltaA)
            call diagonalize_Hermitian_matrix(matdim,Ham,matvec,matval)
            do ie0 = 1,hn
                                          !  u               v
                new_deltaA = new_deltaA + matvec(1,ie0) * matvec(2,ie0) * tanh(matval(ie0)/(2.0 * kbt))
            end do
        end do
        ! 自洽得到新的序参(归一化处理)
        new_deltaA = U0 * new_deltaA * delta_k**2    ! 吸引相互作用才能自洽出超导     \Delta=-U/Nk\sum_{k}\frac{\Delta}{2E(k)}\tanh(E(k)/(2kbT))
        ! 给定差异来停止自洽过程
        diff_A = abs(new_deltaA - old_deltaA)
        ! 重新赋值继续自洽
        old_deltaA = new_deltaA
    end do
    !  返回自洽收敛后的序参量
    delta_A = new_deltaA
    return
end subroutine
!==============================================================================================================================================================
subroutine Fermi_surface()
    use code_param
    integer ikx,iky,ik0,ie
    real(dp) kx,ky,mateigval_1(hnn)
    complex(dp) Ham_up(hnn,hnn),Ham_down(hnn,hnn),mateigvec(hnn,hnn)
    open(31,file = "FS.dat")
    do ik0 = 1,numk_bz
        kx = BZklist(1,ik0)
        ky = BZklist(2,ik0)
        call matset_Normal(kx,ky,Ham_up,Ham_down)
        call diagonalize_Hermitian_matrix(hnn,Ham_up,mateigvec,mateigval_1)
        do ie = 1,hnn
            if (abs(mateigval_1(ie)) < 1e-2)then  ! 给定能量确定费米面
                write(31,"(10F20.8)")kx,ky
            end if
        end do
    end do
    close(31)
    return
end subroutine
!==============================================================================================================================================================
subroutine matset_BdG_SC(kx, ky, Ham_BdG, delta_A)
    ! 自由电子气 BdG 哈密顿量
    use code_param
    implicit none
    real(dp), intent(in) :: kx, ky  ! 最近邻 & 次近邻矢量
    complex(dp), intent(inout) :: Ham_BdG(hn, hn), delta_A

    ! Initialize Hamiltonian to zero
    Ham_BdG = 0.0

    ! Normal part (particle & hole)
    Ham_BdG(1,1) =  t1 * (cos(kx) + cos(ky)) - mu   ! particle
    Ham_BdG(2,2) = -( t1 * (cos(kx) + cos(ky)) - mu)  ! hole

    ! Pairing
    Ham_BdG(1,2) = delta_A
    Ham_BdG(2,1) = conjg(delta_A)

    return
end subroutine matset_BdG_SC
!==============================================================================================================================================================
subroutine matset_Normal(kx,ky,Ham_up,Ham_down)
    !  正常态哈密顿量构建
    !  矩阵赋值,返回Ham_up & Ham_down
    use code_param
    implicit none
    real(dp) kx,ky
    complex(dp) Ham_up(hnn,hnn),Ham_down(hnn,hnn)
    !--------------------
    ! Spin-up
    Ham_up = 0.0
    ! s_0
    Ham_up(1,1) = t1 * (cos(kx) + cos(ky)) - mu
    !--------------------
    !H_{\ua}(k) = H_{\down}(k)
    ! Spin-down
    Ham_down = 0.0
    Ham_down = Ham_up
    return
end subroutine
!================================================================================================================================================================================================
! function fermi(ek)
!     !  万万不能直接用费米分布函数，会存在浮点溢出
!     use code_param,only:dp,kbt
!     real(dp) fermi,ek
!     fermi = 1.0/(exp(ek/kbt) + 1) 
!     return
! end 
! function fermi(ek)
!     !  万万不能直接用费米分布函数，会存在浮点溢出
!     use code_param
!     real(dp) fermi,ek
!     if(ek/kbt > -engcut .and. ek/kbt < engcut) fermi = 1.0/(exp(ek/kbt) + 1) 
!     if(ek/kbt < -engcut) fermi = 1.0
!     if(ek/kbt > engcut) fermi = 0.0
!     return
! end 
real(dp) function fermi(ek)
    ! 零温下的分布函数
    use code_param
    implicit none
    real(dp), intent(in) :: ek

    if (ek < 0.0) then
        fermi = 1.0
    else
        fermi = 0.0
    end if
end function fermi
!============================================================================================================================
function Gaussian_broadening(energy) 
    use code_param
    implicit none
    real(dp), intent(in) :: energy 
    real(dp) Gaussian_broadening
    Gaussian_broadening = exp(-(energy**2) / (2.0 * sigma**2)) / &
            (sigma * sqrt(2.0 * pi))
  end function Gaussian_broadening
!============================================================================================================================
subroutine squareBZ()
    ! 构建四方BZ
    use code_param
    integer ikx,iky,i0
    ! 对于四方点阵,BZ的点数可以直接确定
    numk_bz = (2 * kn)**2   
    allocate(BZklist(2,numk_bz))
    i0 = 0
    do ikx = -kn,kn - 1
        do iky = -kn,kn - 1
            i0 = i0 + 1
            BZklist(1,i0) = pi * ikx/(1.0 * kn)
            BZklist(2,i0) = pi * iky/(1.0 * kn) 
        end do
    end do
    return  
end subroutine
!==============================================================================================================================================================
subroutine diagonalize_complex_matrix(matdim,matin,matout,mateigval)
    ! 对角化一般复数矩阵,这里的本征值是个复数
    ! matin 输入矩阵   matout 本征矢量    mateigval  本征值
    integer matdim,LDA,LDVL,LDVR,LWMAX,INFO,LWORK
    complex,intent(in)::matin(matdim,matdim)
    complex,intent(out)::matout(matdim,matdim)
    complex,intent(out)::mateigval(matdim)
    REAL,allocatable::RWORK(:)
    complex,allocatable::WORK(:)
    complex,allocatable::VL(:,:)
    complex,allocatable::VR(:,:)
    LDA = matdim
    LDVL = matdim
    LDVR = matdim
    LWMAX = 2 * matdim + matdim**2
    ! write(*,*)matin
    allocate(RWORK(2 * matdim))
    allocate(VL(LDVL,matdim))
    allocate(VR(LDVR, matdim))
    allocate(WORK(LWMAX))
    matout = matin

    LWORK = -1
    call cgeev( 'V', 'N', matdim, matout, LDA, mateigval, VL, LDVL,VR, LDVR, WORK, LWORK, RWORK, INFO)
    LWORK = MIN( LWMAX, INT( WORK( 1 ) ) )
    call cgeev( 'V', 'N', matdim, matout, LDA, mateigval, VL, LDVL,VR, LDVR, WORK, LWORK, RWORK, INFO )
    IF( INFO.GT.0 ) THEN
       WRITE(*,*)'The algorithm failed to compute eigenvalues.'
    !    STOP
    END IF
    matout = VL
    return
end subroutine diagonalize_complex_matrix
!================================================================================================================================================================================================
subroutine Matrix_Inv(matdim,matin,matout)
    ! 矩阵求逆 
    implicit none
    integer matdim,dp,info
    parameter(dp = kind(1.0))
    complex(dp),intent(in) :: matin(matdim,matdim)
    complex(dp):: matout(size(matin,1),size(matin,2))
    real(dp):: work2(size(matin,1))            ! work2 array for LAPACK
    integer::ipiv(size(matin,1))     ! pivot indices
    ! Store matin in matout to prevent it from being overwritten by LAPACK
    matout = matin
    ! SGETRF computes an LU factorization of a general M - by - N matrix A
    ! using partial pivoting with row interchanges .
    call CGETRF(matdim,matdim,matout,matdim,ipiv,info)
    ! if (info.ne.0) stop 'Matrix is numerically singular!'
    if (info.ne.0)  write(*,*)'Matrix is numerically singular!'
    ! SGETRI computes the inverse of a matrix using the LU factorization
    ! computed by SGETRF.
    call CGETRI(matdim,matout,matdim,ipiv,work2,matdim,info)
    ! if (info.ne.0) stop 'Matrix inversion failed!'
    if (info.ne.0) write(*,*)'Matrix inversion failed!'
    return
end subroutine Matrix_Inv
!================================================================================================================================================================================================
subroutine diagonalize_Hermitian_matrix(matdim,matin,matout,mateigval)
    !  厄米矩阵对角化
    ! matin 输入矩阵   matout 本征矢量    mateigval  本征值
    integer matdim
    integer lda0,lwmax0,lwork,lrwork,liwork,info
    complex matin(matdim,matdim),matout(matdim,matdim)
    real mateigval(matdim)
    complex,allocatable::work(:)
    real,allocatable::rwork(:)
    integer,allocatable::iwork(:)
    !-----------------
    lda0 = matdim
    lwmax0 = 2 * matdim + matdim**2
    allocate(work(lwmax0))
    allocate(rwork(1 + 5 * matdim + 2 * matdim**2))
    allocate(iwork(3 + 5 * matdim))
    matout = matin
    lwork = -1
    liwork = -1
    lrwork = -1
    call cheevd('V','U',matdim,matout,lda0,mateigval,work,lwork ,rwork,lrwork,iwork,liwork,info)
    lwork = min(2 * matdim + matdim**2, int( work( 1 ) ) )
    lrwork = min(1 + 5 * matdim + 2 * matdim**2, int( rwork( 1 ) ) )
    liwork = min(3 + 5 * matdim, iwork( 1 ) )
    call cheevd('V','U',matdim,matout,lda0,mateigval,work,lwork,rwork,lrwork,iwork,liwork,info)
    if( info .GT. 0 ) then
        open(11,file = "mes.dat",status = "unknown")
        write(11,*)'The algorithm failed to compute eigenvalues.'
        close(11)
    end if
    return
end subroutine diagonalize_Hermitian_matrix
!==============================================================================================================================================================
subroutine diagonalize_real_matrix(matdim,matin,matout,mateigval)
    ! 对角化一般复数矩阵,这里的本征值是个复数
    ! matin 输入矩阵   matout 本征矢量    mateigval  本征值
    integer matdim,LDA,LDVL,LDVR,LWMAX,INFO,LWORK
    real,intent(in)::matin(matdim,matdim)
    real,intent(out)::matout(matdim,matdim)
    complex,intent(out)::mateigval(matdim)
    real valre(matdim),valim(matdim)
    real,allocatable::WORK(:)
    real,allocatable::VL(:,:)
    real,allocatable::VR(:,:)
    LDA = matdim
    LDVL = matdim
    LDVR = matdim
    LWMAX = 2 * matdim + matdim**2
    ! write(*,*)matin
    allocate(VL(LDVL,matdim))
    allocate(VR(LDVR, matdim))
    allocate(WORK(LWMAX))
    matout = matin

    LWORK = -1
    call sgeev( 'V', 'N', matdim, matout, LDA, valre,valim, VL, LDVL,VR, LDVR, WORK, LWORK, INFO)
    LWORK = MIN( LWMAX, INT( WORK( 1 ) ) )
    call sgeev( 'V', 'N', matdim, matout, LDA, valre,valim, VL, LDVL,VR, LDVR, WORK, LWORK, INFO)
    IF( INFO.GT.0 ) THEN
       WRITE(*,*)'The algorithm failed to compute eigenvalues.'
    !    STOP
    END IF
    do info = 1,matdim
        mateigval(info) = valre(info) + im * valim(info)
    end do
    matout = VL
    return
end subroutine diagonalize_real_matrix
